{ **************************************************************************** }
{ *                                                                          * }
{ * MONKEY (VTech, Time & Fun) Simulator                                     * }
{ * by Luca "MADrigal" Antignano, December 2007 (revision: August 2017)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit manual;
{ this unit contains the instructions manual browser implementation            }

interface

uses
  Windows, Forms, Graphics, Controls, Classes, ExtCtrls;

type
  TManualForm = class(TForm)
    sh_page_top:    TShape;

    im_page_top:    TImage;

    im_page_01:     TImage;
    im_page_02:     TImage;
    im_page_03:     TImage;
    im_page_04:     TImage;
    im_page_05:     TImage;
    im_page_06:     TImage;
    im_page_07:     TImage;
    im_page_08:     TImage;

    // controls
    btn_left_down:  TImage;         btn_left_up:  TImage;
    btn_right_down: TImage;         btn_right_up: TImage;

    btn_close_down: TImage;         btn_close_up: TImage;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate            (Sender: TObject);
    procedure FormShow              (Sender: TObject);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_Page;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure sh_page_topMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure sh_page_topMouseMove  (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_left_upMouseDown  (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_upMouseUp    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_upMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_upMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_close_upMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_close_upMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown           (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp             (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_left_down;        procedure Inp_left_up;
    procedure Inp_right_down;       procedure Inp_right_up;
    procedure Inp_esc_down;         procedure Inp_esc_up;
  end;

const
  MANUALPAGES = 8;

var
{ * main objects * }
  ManualForm:       TManualForm;                                                // main form
  iMouseX, iMouseY: integer;                                                    // cursor coordinates (to drag main form)

{ * display arrays of images * }
  aIm_pages:        array[1..MANUALPAGES] of TImage;                            // instructions manual pages

{ * integers and booleans * }
  iActivePage                                                                   // currently displayed page
                  : integer;

implementation

uses
{}unit1, gfxinit;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.FormCreate(Sender: TObject);
{ pages sprites are associated to the array of images, needed to "index" them  }
begin
  aIm_pages[1] := im_page_01;
  aIm_pages[2] := im_page_02;
  aIm_pages[3] := im_page_03;
  aIm_pages[4] := im_page_04;
  aIm_pages[5] := im_page_05;
  aIm_pages[6] := im_page_06;
  aIm_pages[7] := im_page_07;
  aIm_pages[8] := im_page_08;

  // loads graphics resources and initializes the instructions manual graphics
  Gfx_Manual_Init;

{}// visualization
{}Position := poScreenCenter;
{}FormStyle := fsStayOnTop;
{}
{}// variables init
{}DoubleBuffered := true;                                                       // enables double buffering for rendering of this form
end;





procedure TManualForm.FormShow(Sender: TObject);
{ displays the current page                                                    }
begin
  Vid_Show_Page;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.Vid_Show_Page;
{ displays active page into the "container" image                              }
begin
  im_page_top.Picture := aIm_pages[iActivePage].Picture;

  case iActivePage of
    1: begin
         btn_left_down.Visible := false;
         btn_left_up.Visible := false;
       end;
    MANUALPAGES: begin
         btn_right_down.Visible := false;
         btn_right_up.Visible := false;
       end;
    else
       begin
         btn_left_down.Visible := true;
         btn_left_up.Visible := true;
         btn_right_down.Visible := true;
         btn_right_up.Visible := true;
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.sh_page_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
begin
  if Button = mbLeft then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TManualForm.sh_page_topMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
begin
  if ssLeft in Shift then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TManualForm.btn_left_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TManualForm.btn_left_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TManualForm.btn_right_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TManualForm.btn_right_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up
end;





procedure TManualForm.btn_close_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ close button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_esc_down;
end;





procedure TManualForm.btn_close_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ close button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_esc_up;
end;





procedure TManualForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  case Key of
    VK_LEFT:   if iActivePage > 1           then Inp_left_down;
    VK_RIGHT:  if iActivePage < MANUALPAGES then Inp_right_down;
    VK_ESCAPE: Inp_esc_down;
  end; // case..of
end;





procedure TManualForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  case Key of
    VK_LEFT:   if iActivePage > 1           then Inp_left_up;
    VK_RIGHT:  if iActivePage < MANUALPAGES then Inp_right_up;
    VK_ESCAPE: Inp_esc_up;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TManualForm.Inp_left_down;
{ left button: PRESS                                                           }
begin
  btn_left_up.Visible := false;
end;





procedure TManualForm.Inp_left_up;
{ left button: RELEASE                                                         }
begin
  Dec(iActivePage); Vid_Show_Page;
end;





procedure TManualForm.Inp_right_down;
{ right button: PRESS                                                          }
begin
  btn_right_up.Visible := false;
end;





procedure TManualForm.Inp_right_up;
{ right button: RELEASE                                                        }
begin
  Inc(iActivePage); Vid_Show_Page;
end;





procedure TManualForm.Inp_esc_down;
{ close button: PRESS                                                          }
begin
  btn_close_up.Visible := false;
end;





procedure TManualForm.Inp_esc_up;
{ close button: RELEASE                                                        }
begin
  btn_close_up.Visible := true;

  Close;
end;





initialization
  iActivePage := 1;





end.
