{ **************************************************************************** }
{ *                                                                          * }
{ * MONKEY (VTech, Time & Fun) Simulator                                     * }
{ * by Luca "MADrigal" Antignano, December 2007 (revision: August 2017)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;    im_background_screen_top: TImage;

    btn_game_a_down:          TImage;    btn_game_a_top:           TImage;
    btn_game_b_down:          TImage;    btn_game_b_top:           TImage;
    btn_time_down:            TImage;    btn_time_top:             TImage;
                                         btn_set_top:              TImage;

    btn_left_down:            TImage;    btn_left_top:             TImage;
    btn_right_down:           TImage;    btn_right_top:            TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_1: TImage; im_panel_time_2: TImage; im_panel_alarm: TImage; im_panel_chrono: TImage; im_panel_date: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_bell: TImage; 

    im_panel_miss_01: TImage; im_panel_miss_02: TImage; im_panel_miss_03: TImage; im_panel_miss_04: TImage; im_panel_miss_05: TImage;
    im_panel_miss_06: TImage; im_panel_miss_07: TImage; im_panel_miss_08: TImage; im_panel_miss_09: TImage; im_panel_miss_10: TImage;

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_hero_1_1: TImage;      im_hero_2_1: TImage;      im_hero_3_1: TImage;
    im_hero_1_2: TImage;      im_hero_2_2: TImage;      im_hero_3_2: TImage;
    im_hero_1_3: TImage;      im_hero_2_3: TImage;      im_hero_3_3: TImage;

    im_item_1_1: TImage;      im_item_2_1: TImage;      im_item_3_1: TImage;
    im_item_1_2: TImage;      im_item_2_2: TImage;      im_item_3_2: TImage;
    im_item_1_3: TImage;      im_item_2_3: TImage;      im_item_3_3: TImage;
    im_item_1_4: TImage;      im_item_2_4: TImage;      im_item_3_4: TImage;
    im_item_1_5: TImage;      im_item_2_5: TImage;      im_item_3_5: TImage;
    im_item_1_got: TImage;    im_item_2_got: TImage;    im_item_3_got: TImage;
    im_item_1_miss: TImage;   im_item_2_miss: TImage;   im_item_3_miss: TImage;
    im_item_1_target: TImage; im_item_2_target: TImage; im_item_3_target: TImage;

{ * TIMERS                                                                   * }
    Timer_Alarm:      TTimer;
    Timer_Clock:      TTimer;
    Timer_Date:       TTimer;
    Timer_Game:       TTimer;
    Timer_Game_Over:  TTimer;
    Timer_Game_Start: TTimer;
    Timer_Hit:        TTimer;
    Timer_Reprise:    TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                 (Sender: TObject);
    procedure FormActivate               (Sender: TObject);
    procedure FormClose                  (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All               (b: boolean);

    procedure Vid_Show_All_Hero          (b: boolean);
    procedure Vid_Show_All_Item          (b: boolean);
    procedure Vid_Show_All_Item_Got      (b: boolean);
    procedure Vid_Show_All_Item_Miss     (b: boolean);
    procedure Vid_Show_All_Item_Target   (b: boolean);

    procedure Vid_Show_Hero              (var p: integer; q: integer);

    procedure Vid_Show_Panel_Miss        (var p: integer);
    procedure Vid_Show_Panel_Score       (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                   (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove     (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown    (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                  (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_game_a_down;           procedure Inp_game_a_up;
    procedure Inp_game_b_down;           procedure Inp_game_b_up;
    procedure Inp_time_down;             procedure Inp_time_up;

    procedure Inp_left_down;             procedure Inp_left_up;
    procedure Inp_right_down;            procedure Inp_right_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;
    procedure Gam_Set_Mode;
    procedure Gam_Set_Level;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_AlarmTimer           (Sender: TObject);
    procedure Timer_ClockTimer           (Sender: TObject);
    procedure Timer_DateTimer            (Sender: TObject);
    procedure Timer_GameTimer            (Sender: TObject);
    procedure Timer_Game_OverTimer       (Sender: TObject);
    procedure Timer_Game_StartTimer      (Sender: TObject);
    procedure Timer_HitTimer             (Sender: TObject);
    procedure Timer_RepriseTimer         (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup             (Sender: TObject);
    procedure pm_1_aboutClick            (Sender: TObject);
    procedure pm_2_readmeClick           (Sender: TObject);
    procedure pm_3_whatsnewClick         (Sender: TObject);
    procedure pm_4_manualClick           (Sender: TObject);
    procedure pm_6_configClick           (Sender: TObject);
    procedure pm_7_keysClick             (Sender: TObject);
    procedure pm_8_hiscoreClick          (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick     (Sender: TObject);

    procedure la_menu_itemMouseEnter     (Sender: TObject);
    procedure la_menu_itemMouseLeave     (Sender: TObject);
    procedure la_menu_itemMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu      (b: boolean);
  end;

const
  APP_NAME    = 'Monkey (VTech, Time & Fun) Simulator';                         // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version 5.00 * Final *';                                       // current version (appears on Splash screen)
{}APP_DATE    = 'Released by Luca "MADrigal" Antignano on 4th December, 2017';  // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:lucantignano@gmail.com';                                // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://www.madrigaldesign.it/sim/';                            // link to the website

  REG_KEY     = 'Software\MADrigal\Monkey50';                                   // registry key

  HINT_CLICK  = 'click or press';
  HINT_GAME_A = 'to start game A (hold to view highscore)';
  HINT_GAME_B = 'to start game B (hold to view highscore)';
  HINT_TIME   = 'to switch mode';
  HINT_SET    = 'button disabled';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_gameover,
  pfs_got,
  pfs_miss,
  pfs_move,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // numbers for score/time
  aIm_panel_miss:          array[1..10]      of TImage;                         // misses/chrono counter sprites

  aIm_hero:                array[1..3, 1..3] of TImage;                         // main character sprites
  aIm_item:                array[1..3, 1..5] of TImage;                         // moveable items
  aIm_item_miss,                                                                // missed items
  aIm_item_got,                                                                 // got items
  aIm_item_target:         array[1..3]       of TImage;                         // target items

{ * arrays of numbers *}
  aItem:                   array[1..3, 1..5] of integer;                        // 0=empty; 1=item moving; 2=item just got
  aItemMiss:               array[1..3]       of integer;                        // 0=empty; 1=item missed
  aItemGot:                array[1..3]       of integer;                        // 0=empty; 1=about to be removed

{ * arrays of mixed types * }
  aKeys:                   array[1..5] of record                                // keys management (values)
                                            int:  integer;
                                            word: word;
                                           end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL, 1=time, etc.
  iScore,                                                                       // 0-9999
  iHiScoreA,                                                                    // 0-9999
  iHiScoreB,                                                                    // 0-9999

  iMiss,                                                                        // 0-5
  iHeroPos,                                                                     // 1-3
  iMaxOnscreen,                                                                 // 1-? (maximum allowed moveable items on screen simultaneously)

  iLoop                                                                         // 1-3 for "TimerGame", 1-? for other routines
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bKeyPressed,                                                                  // true while left/right arrow keys are being kept pressed
  bGameAPressed,                                                                // true while 'Game A' button is being kept pressed
  bGameBPressed,                                                                // true while 'Game B' button is being kept pressed
  bControllers                                                                  // true while player can actually operate the main character
                         : boolean;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel (alarm, time/score, game a/b, etc.)
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  aIm_panel_miss[1] := im_panel_miss_01; aIm_panel_miss[2] := im_panel_miss_02; aIm_panel_miss[3] := im_panel_miss_03; aIm_panel_miss[4] := im_panel_miss_04; aIm_panel_miss[5] := im_panel_miss_05;
  aIm_panel_miss[6] := im_panel_miss_06; aIm_panel_miss[7] := im_panel_miss_07; aIm_panel_miss[8] := im_panel_miss_08; aIm_panel_miss[9] := im_panel_miss_09; aIm_panel_miss[10] := im_panel_miss_10;

  // on-screen items
  aIm_hero[1, 1] := im_hero_1_1; aIm_hero[1, 2] := im_hero_1_2; aIm_hero[1, 3] := im_hero_1_3;
  aIm_hero[2, 1] := im_hero_2_1; aIm_hero[2, 2] := im_hero_2_2; aIm_hero[2, 3] := im_hero_2_3;
  aIm_hero[3, 1] := im_hero_3_1; aIm_hero[3, 2] := im_hero_3_2; aIm_hero[3, 3] := im_hero_3_3;

  aIm_item[1, 1] := im_item_1_1; aIm_item[1, 2] := im_item_1_2; aIm_item[1, 3] := im_item_1_3; aIm_item[1, 4] := im_item_1_4; aIm_item[1, 5] := im_item_1_5;
  aIm_item[2, 1] := im_item_2_1; aIm_item[2, 2] := im_item_2_2; aIm_item[2, 3] := im_item_2_3; aIm_item[2, 4] := im_item_2_4; aIm_item[2, 5] := im_item_2_5;
  aIm_item[3, 1] := im_item_3_1; aIm_item[3, 2] := im_item_3_2; aIm_item[3, 3] := im_item_3_3; aIm_item[3, 4] := im_item_3_4; aIm_item[3, 5] := im_item_3_5;

  aIm_item_miss[1] := im_item_1_miss;     aIm_item_miss[2] := im_item_2_miss;     aIm_item_miss[3] := im_item_3_miss;

  aIm_item_got[1] := im_item_1_got;       aIm_item_got[2] := im_item_2_got;       aIm_item_got[3] := im_item_3_got;

  aIm_item_target[1] := im_item_1_target; aIm_item_target[2] := im_item_2_target; aIm_item_target[3] := im_item_3_target;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 39);                             // "Left"   (default: "LEFT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 40);                             // "Right"  (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 5 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_set_top.Hint    := HINT_SET;                                                       // "Set"

  btn_left_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];                     // "Left"
  btn_right_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bKeypressed := false;                                                         // no key is currently being kept pressed
  bGameAPressed := false;                                                       // 'Game A' button is not currently pressed
  bGameBPressed := false;                                                       // 'Game B' button is not currently pressed
  bControllers := false;                                                        // prevents main character movement

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_gameover); FSOUND_Sample_Free(pfs_got);
  FSOUND_Sample_Free(pfs_miss);     FSOUND_Sample_Free(pfs_move);
  FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides nearly all sprites (except score panel)                       }
  var
    i: integer;
begin
  im_panel_game_a.Visible     := b;
  im_panel_game_b.Visible     := b;
  im_panel_time_1.Visible     := b;
  im_panel_time_2.Visible     := b;
  im_panel_time_am.Visible    := b;
  im_panel_time_pm.Visible    := b;
  im_panel_bell.Visible       := b;
  im_panel_alarm.Visible      := b;
  im_panel_date.Visible       := b;
  im_panel_chrono.Visible     := b;
  im_panel_time_colon.Visible := b;

  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  i := 8888; Vid_Show_Panel_Score(i, b);
  i := Ord(b) * 10; Vid_Show_Panel_Miss(i);

  Vid_Show_All_Hero(b);
  Vid_Show_All_Item(b);
  Vid_Show_All_Item_Miss(b);
  Vid_Show_All_Item_Got(b);
  Vid_Show_All_Item_Target(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all main character sprites                                    }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 3 do
      aIm_hero[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item(b: boolean);
{ displays/hides all 'moveable' items                                          }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 5 do
      aIm_item[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item_Got(b: boolean);
{ displays/hides all 'got' items                                               }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_item_got[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item_Miss(b: boolean);
{ displays/hides all 'missed' items                                            }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_item_miss[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Item_Target(b: boolean);
{ displays/hides all 'got' items                                               }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_item_target[i].Visible := b;
end;





procedure TForm1.Vid_Show_Hero(var p: integer; q: integer);
{ shows/hides main character in 'p' position, in 'q' status                    }
  var
    i: integer;
begin
  case q of
    0: for i := 1 to 3 do aIm_hero[p, i].Visible := false;                                                          // hides hero completely
    1: begin aIm_hero[p, 1].Visible := true; aIm_hero[p, 2].Visible := true;  aIm_hero[p, 3].Visible := false; end; // hero_x_2 on
    2: begin aIm_hero[p, 1].Visible := true; aIm_hero[p, 2].Visible := false; aIm_hero[p, 3].Visible := true;  end; // hero_x_3 on
  end; // case..of
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays as many sprites as p on the misses counter (0=nothing visible)      }
  var
    i: integer;
begin
  for i := 1 to 10 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 10000 do                                                           // displays "10000" as "...0"
    Dec(t, 10000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                               // display
      if t > 999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].Picture else aIm_panel_digit_pos[1].Picture := nil; // thousands
      if t > 99  then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].Picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t > 9   then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].Picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].Picture;                                      // units
    end
  else                                                                                  // hide
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y:Integer);
{ game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ mode button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ mode button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_left_down   else                              // Left
  if Key = aKeys[5].word then Inp_right_down;                                   // Right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_left_up   else                                // Left
  if Key = aKeys[5].word then Inp_right_up;                                     // Right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if not bGameAPressed then
    begin
      bGameAPressed := true;

      if (iMode = 0) then                                                       // ACL ->
        begin
          iMode := 1;                                                           // -> Time 1
          Gam_Set_Mode;
        end
      else
        if ( (iMode > 0) and (iMode < 6) ) or (iMode = 10) then                 // Time1, Time 2, Alarm, Chrono, Date, Game Over ->
          begin
            iMode := 6;                                                         // -> HiScore A
            Gam_Set_Mode;
          end;
    end;
end;





procedure TForm1.Inp_game_a_up;
{ game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;
  bGameAPressed := false;

  if (iMode = 6) and (not Timer_Game_Start.Enabled) then                        // HiScore A and plays "start" sound sample ->
    begin
      iMode := 8;                                                               // -> Game A
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_game_b_down;
{ game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if not bGameBPressed then
    begin
      bGameBPressed := true;

      if (iMode = 0) then                                                       // ACL
        begin
          iMode := 1;                                                           // -> Time 1
          Gam_Set_Mode;
        end
      else
        if ( (iMode > 0) and (iMode < 6) ) or (iMode = 10) then                 // Time1, Time 2, Alarm, Chrono, Date, Game Over ->
          begin
            iMode := 7;                                                         // ->  HiScore B
            Gam_Set_Mode;
          end;
    end;
end;





procedure TForm1.Inp_game_b_up;
{ game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;
  bGameBPressed := false;

  if (iMode = 7) and (not Timer_Game_Start.Enabled) then                        // HiScore B and plays "start" sound sample ->
    begin
      iMode := 9;                                                               // -> Game B
      Gam_Set_Mode;
    end;
end;





procedure TForm1.Inp_time_down;
{ mode button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if ( (iMode >= 0) and (iMode < 6) ) or (iMode = 10) then                  // any other mode except hiscore/game modes ->
        begin
          Inc(iMode);                                                           // -> Time 1 -> Time 2 -> Alarm -> Date -> Chrono ->

          if iMode > 5 then                                                     // resets to Time 1 if mode higher than 5
            iMode := 1;

          Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_time_up;
{ mode button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_left_down;
{ left button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if bControllers and (iHeroPos > 1) then
        begin
          Vid_Show_Hero(iHeroPos, 0);                                           // hides main character
          Dec(iHeroPos);                                                        // updates coordinates
          Vid_Show_Hero(iHeroPos, 1);                                           // displays main character
        end;
    end;
end;





procedure TForm1.Inp_left_up;
{ left button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;
  bKeyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ right button: PRESS                                                          }
begin
  btn_right_down.Visible := true;

  if not bKeyPressed then
    begin
      bKeyPressed := true;

      if bControllers and (iHeroPos < 3) then
        begin
          Vid_Show_Hero(iHeroPos, 0);                                           // hides main character
          Inc(iHeroPos);                                                        // updates coordinates
          Vid_Show_Hero(iHeroPos, 1);                                           // displays main character
        end;
    end;
end;





procedure TForm1.Inp_right_up;
{ right button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;
  bKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Alarm.Enabled      := false;
  Timer_Clock.Enabled      := false;
  Timer_Date.Enabled       := false;
  Timer_Game.Enabled       := false;
  Timer_Game_Over.Enabled  := false;
  Timer_Game_Start.Enabled := false;
  Timer_Hit.Enabled        := false;
  Timer_Reprise.Enabled    := false;
end;





procedure TForm1.Gam_Set_Level;
{ sets number of maximum moveable items on screen simultaneously, and value    }
{ for the TimerGame interval, making game faster/slower depending on which     }
{ game is played (A or B) and current score                                    }
  var
    i: integer;
begin
  i := iScore;

  while i > 199 do
    Dec(i, 200);

  case iMode of
    8: begin                                                                    // Game A
         // maximum moveable items on screen
         case i of
           0..3:     iMaxOnscreen := 1;                                         //   0....3
           4..49:    iMaxOnscreen := 3;                                         //   4...49
           50..99:   iMaxOnscreen := 4;                                         //  50...99
           100..199: iMaxOnscreen := 5;                                         // 100..199
         end; // case..of

         // game speed
         case i of
           0..99:    Timer_Game.Interval := 250;                                //   0...99
           100..199: Timer_Game.Interval := 215;                                // 100..199
         end; // case..of
       end;
    9: begin                                                                    // Game B
         // maximum moveable items on screen
         case i of
           0..3:     iMaxOnscreen := 1;                                         //   0....3
           4..49:    iMaxOnscreen := 4;                                         //   4...49
           50..99:   iMaxOnscreen := 5;                                         //  50...99
           100..199: iMaxOnscreen := 6;                                         // 100..199
         end; // case..of

         // game speed
         case i of
           0..99:    Timer_Game.Interval := 215;                                //   0...99
           100..199: Timer_Game.Interval := 185;                                // 100..199
         end; // case..of
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/ACL mode, depending on the value of the iMode variable  }
  var
    i, j: integer;
begin
  case iMode of
    0: Vid_Show_All(true);                                                      // ACL
    1: begin                                                                    // TIME 1
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_time_1.Visible := true;                                               // displays "time 1"

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;                                                   // clock starts ticking...
       end;
    2: begin                                                                    // TIME 2
         im_panel_time_1.Visible := false;                                              // hides "time 1"
         im_panel_time_2.Visible := true;                                               // displays "time 2"
       end;
    3: begin                                                                    // ALARM
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_alarm.Visible := true;                                                // displays "alarm"
         i := 1200; Vid_Show_Panel_Score(i, true);                                      // 12:00

         Timer_Alarm.Interval := 1;
         Timer_Alarm.Enabled := true;                                                   // clock starts ticking... (colon only)
       end;
    4: begin                                                                    // CHRONO
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_chrono.Visible := true;                                               // displays "chrono"
         im_panel_time_colon.Visible := true;                                           // displays colon

         for i := 1 to 4 do
           aIm_panel_digit_pos[i].Picture := aim_panel_number[0].Picture;               // 00:00
       end;
    5: begin                                                                    // DATE
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.
         im_panel_date.Visible := true;                                                 // displays "date"

         Timer_Date.Interval := 1;
         Timer_Date.Enabled := true;                                                    // clock starts ticking...
       end;
    6, 7: begin                                                                    // HISCORE A /B
         Gam_Stop_All_Timers;                                                           // stops all timers
         Vid_Show_All(false);                                                           // hides all items, chars, etc.

         case iMode of
           6: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of

         iHeroPos := 2; Vid_Show_Hero(iHeroPos, 1);                                     // displays main character in x=2
         Vid_Show_All_Item_Target(true);                                                // displays all target items
         Snd_Play(pfs_start);                                                           // plays "Start" tune

         Timer_Game_Start.Interval := 1800;
         Timer_Game_Start.Enabled := true;                                              // enables "Game Start" timer, then game starts
       end;
    8, 9: begin                                                                 // GAME A / B
         iScore := 0; Vid_Show_Panel_Score(iScore, true);                               // displays score (=0)
         iMiss := 0; Vid_Show_Panel_Miss(iMiss);                                        // display misses (=0)
         Gam_Set_Level;                                                                 // sets game speed, difficulty etc.

         for i:=1 to 3 do                                                               // clears moveable items array
           begin
             for j := 1 to 5 do
               aItem[i, j] := 0;

             aItemGot[i] := 0;                                                          // clears "got" items array
             aItemMiss[i] := 0;                                                         // clears "missed" items array
           end;

         bControllers := true;                                                          // enables main character control

         iLoop := 1;                                                                    // initializes game loop
         Timer_Game.Enabled := true;                                                    // game starts
       end;
   10: begin                                                                    // GAME OVER
         // does nothing...
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_AlarmTimer(Sender: TObject);
{ time colon blinks at every 500 milliseconds                                  }
begin
  im_panel_time_colon.Visible := not im_panel_time_colon.Visible;               // colon animation

  Timer_Alarm.Interval := 500;
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, and displays it                 }
{ time colon blinks at every 500 milliseconds                                  }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  im_panel_time_colon.Visible := not im_panel_time_colon.Visible;               // colon animation

  Timer_Clock.Interval := 500;
end;





procedure TForm1.Timer_DateTimer(Sender: TObject);
{ transforms current date into "sprites" form, and displays it                 }
{ time colon blinks at every 500 milliseconds                                  }
  var
    present:          TDateTime;
    day, month, year: Word;
begin
  present := Now;                                                               // gets system date...
  DecodeDate(present, year, month, day);                                        // and decodes it

  if month > 9 then                                                             // month: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[month div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[month mod 10].Picture;     // month: units

  if day > 9 then                                                               // day: tens
    aIm_panel_digit_pos[3].Picture := aIm_panel_number[day div 10].Picture
  else
    aIm_panel_digit_pos[3].Picture := nil;

  aIm_panel_digit_pos[4].Picture := aIm_panel_number[day mod 10].Picture;       // day: units

  im_panel_time_colon.Visible := not im_panel_time_colon.Visible;               // colon animation

  Timer_Date.Interval := 500;
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ sets Game A or Game B modes after Game A or Game B buttons are released      }
begin
  if ((iMode = 6) and (not bGameAPressed))
    or ((iMode = 7) and (not bGameBPressed)) then
    begin
      Inc(iMode, 2);                                                            // HiScore A -> Game A or HiScore B -> Game B
      Gam_Set_Mode;
    end;

  Timer_Game_Start.Enabled := false;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iOnscreen,
    iMistake,
    iGot,
    iMoved:    integer;
begin
  // various initialization (for later use within this procedure)
  iMistake := 0;
  iGot     := 0;
  iMoved   := 0;

  // keeps and then removes "got" items
  for i := 1 to 3 do
    begin
      if aItemGot[i] = 1 then
        aItemGot[i] := 0;

      if aItem[i, 5] = 2 then
        begin
          aItem[i, 5] := 0;
          aItemGot[i] := 1;
        end;
    end;

  // checks for mistakes
  for i := 1 to 3 do
    if (aItem[i, 5] = 1) and (iHeroPos <> i) then
      begin
        aItem[i, 5] := 0;
        aItemMiss[i] := 1;
        iMistake := i;
      end;

  // checks for collisions character-items on certain positions and manages 'hit' animation of main character
  if (iMistake = 0) and (aItem[iHeroPos, 5] = 1) then
    begin
      Vid_Show_Hero(iHeroPos, 2);

      Timer_Hit.Interval := 100;
      Timer_Hit.Enabled := true;

      aItem[iHeroPos, 5] := 2;                                                  // removed at next tick
      Inc(iScore);

      if iScore > 9999 then                                                     // increases score and resets to zero if above 9999
        Dec(iScore, 10000);

      Vid_Show_Panel_Score(iScore, true);
      Gam_Set_Level;                                                            // calculates new speed and difficulty
      iGot := iHeroPos;
    end;

  // moves and generates items
  if iMistake = 0 then
    begin

      // moves items
      for i := 5 downto 2 do
        aItem[iLoop, i] := aItem[iLoop, i - 1];

      aItem[iLoop, 1] := 0;

      // generates new items
      iOnscreen := 0;

      for i := 1 to 3 do                                                        // counts items on screen
        for j := 1 to 5 do
          if aItem[i, j] = 1 then
            Inc(iOnscreen);                                 

      if Random(4) = 0 then                                                     // generates randomly
        aItem[iLoop, 1] := 1
      else
        aItem[iLoop, 1] := 0;

      if iOnscreen = 0 then                                                     // forces generation if none on screen
        aItem[iLoop, 1] := 1;

      if aItem[iLoop, 2] = 1 then                                               // prevents from generating if one just appeared on the same column
        aItem[iLoop, 1] := 0;

      case iLoop of                                                             // prevents from generating if one just appeared on previous column
        1: if aItem[3, 1] = 1 then aItem[iLoop, 1] := 0;
        2: if aItem[1, 1] = 1 then aItem[iLoop, 1] := 0;
        3: if aItem[2, 1] = 1 then aItem[iLoop, 1] := 0;
      end; // case..of

      if iOnscreen >= iMaxOnscreen then                                         // prevents from generating if more onscreen items than allowed
        aItem[iLoop, 1] := 0;

      // gets ready to play "pfMove" sound
      for i := 1 to 5 do
        if aItem[iLoop, i] = 1 then
          Inc(iMoved);
    end;

  // executes miss routine, counts misses and reprises or goes to "game over"
  if iMistake <> 0 then                                                         // checks if any miss just occured
    begin
      bControllers := false;
      Gam_Stop_All_Timers;
      aIm_item_miss[iMistake].visible := true;                                          // renders "missed" item on screen
      aIm_item_target[iMistake].visible := false;                                       // hides "target" item
      Inc(iMiss); Vid_Show_Panel_Miss(iMiss);                                           // updates "misses" panel

      if iMiss < 5 then                                                                 // checks how many misses were done
        begin                                                                                // game continues if misses less than 5
          Timer_Reprise.Interval := 1100;
          Timer_Reprise.Enabled := true;
        end
      else
        begin                                                                                // game is over if misses=5
          iLoop := 0; { becomes 1 before the end of this procedure }
          Timer_Game_Over.Interval := 100;
          Timer_Game_Over.Enabled := true;
        end;
    end
  else
    begin                                                                       // no misses
      Inc(iLoop);

      if iLoop > 3 then                                                         // loops action
        iLoop := 1;                                             
    end;

  // renders moveable items on screen
  for i := 1 to 3 do
    for j := 1 to 5 do
      aIm_item[i, j].visible := (aItem[i, j] > 0);

  // renders 'got' items on screen
  for i := 1 to 3 do
    aIm_item_got[i].visible := (aItemGot[i] = 1);

  // plays correct sound (miss, got, move)
  if iMistake > 0 then
    Snd_Play(pfs_miss)
  else
    if iGot > 0 then
      Snd_Play(pfs_got)
    else
      if iMoved > 0 then
        Snd_Play(pfs_move);
end;





procedure TForm1.Timer_HitTimer(Sender: TObject);
{ main character goes back to "normal" animaton frame                          }
begin
  Vid_Show_Hero(iHeroPos, 1);

  Timer_Hit.Enabled := false;
end;





procedure TForm1.Timer_RepriseTimer(Sender: TObject);
{ game continues after "miss"                                                  }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aItemMiss[i] := 0;

  Vid_Show_All_Item_Miss(false);
  Vid_Show_All_Item_Target(true);
  bControllers := true;

  Timer_Game.Enabled := true;
  Timer_Reprise.Enabled := false;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ game is over: play "Game Over" tune, then freeze for 30 seconds, and finally }
{ change mode to "Time 1", if no button is pressed                             }
begin
  case iLoop of
    1: begin
         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 3400;
       end;
    2: begin
         if (iMode = 8) and (iScore > iHiScoreA) then                           // records high score to system registry (game A)
           begin
             iHiScoreA := iScore;
             iReg.WriteInteger('', 'HiScore1', iHiScoreA);                              // saves to system registry
           end;

         if (iMode = 9) and (iScore > iHiScoreB) then                           // records high score to system registry (game B)
           begin
             iHiScoreB := iScore;
             iReg.WriteInteger('', 'HiScore2', iHiScoreB);                              // saves to system registry
           end;

         iMode := 10;                                                           // sets "Game over" mode
         Gam_Set_Mode;

         Timer_Game_Over.Interval := 30000;
       end;
    3: begin
         iMode := 1;                                                            // changes to "Time 1" mode, 30 seconds after "Game over"
         Gam_Set_Mode;

         Timer_Game_Over.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
